RayTestGeometry

Struct RayTestGeometry 

pub struct RayTestGeometry<Point>
where Point: EuclideanSpace<Scalar = f32>,
{ /* private fields */ }
A type implementing TestGeometry for rays

Implementations

Source

impl<Point> RayTestGeometry<Point>
where Point: EuclideanSpace<Scalar = f32> + VecDim,

pub fn with_system_bounds( system_bounds: Bounds<Point>, origin: Point, direction: Point::Diff, range_min: f32, range_max: f32, ) -> Self
where Point: Debug, Point::Diff: ElementWise + Index<usize, Output = f32> + Debug,

Construct ray test geometry

range_min and range_max may be infinity or negative infinity, in which case the ray will be clamped to system bounds